/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.site.apio.internal.architect.resource;

import com.liferay.apio.architect.pagination.PageItems;
import com.liferay.apio.architect.pagination.Pagination;
import com.liferay.apio.architect.representor.Representor;
import com.liferay.apio.architect.resource.CollectionResource;
import com.liferay.apio.architect.routes.CollectionRoutes;
import com.liferay.apio.architect.routes.ItemRoutes;
import com.liferay.folder.apio.architect.identifier.RootFolderIdentifier;
import com.liferay.person.apio.architect.identifier.PersonIdentifier;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.model.Group;
import com.liferay.portal.kernel.service.GroupService;
import com.liferay.portal.kernel.theme.ThemeDisplay;
import com.liferay.site.apio.architect.identifier.WebSiteIdentifier;
import com.liferay.site.apio.internal.model.GroupWrapper;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;

/**
 * Provides the information necessary to expose <a
 * href="http://schema.org/WebSite">WebSite </a> resources through a web API.
 * The resources are mapped from the internal model {@link Group}.
 *
 * @author Victor Oliveira
 * @author Alejandro Hernández
 */
@Component(immediate = true)
public class WebSiteCollectionResource
	implements CollectionResource<GroupWrapper, Long, WebSiteIdentifier> {

	@Override
	public CollectionRoutes<GroupWrapper, Long> collectionRoutes(
		CollectionRoutes.Builder<GroupWrapper, Long> builder) {

		return builder.addGetter(
			this::_getPageItems, ThemeDisplay.class
		).build();
	}

	@Override
	public String getName() {
		return "web-site";
	}

	@Override
	public ItemRoutes<GroupWrapper, Long> itemRoutes(
		ItemRoutes.Builder<GroupWrapper, Long> builder) {

		return builder.addGetter(
			this::_getGroupWrapper, ThemeDisplay.class
		).build();
	}

	@Override
	public Representor<GroupWrapper> representor(
		Representor.Builder<GroupWrapper, Long> builder) {

		return builder.types(
			"WebSite"
		).identifier(
			Group::getGroupId
		).addBidirectionalModel(
			"interactionService", "webSites", WebSiteIdentifier.class,
			this::_getParentGroupId
		).addBoolean(
			"active", Group::isActive
		).addLinkedModel(
			"author", PersonIdentifier.class, Group::getCreatorUserId
		).addLinkedModel(
			"creator", PersonIdentifier.class, Group::getCreatorUserId
		).addLinkedModel(
			"folder", RootFolderIdentifier.class, Group::getGroupId
		).addLocalizedStringByLocale(
			"description", Group::getDescription
		).addLocalizedStringByLocale(
			"name", Group::getName
		).addRelatedCollection(
			"members", PersonIdentifier.class
		).addString(
			"membershipType", Group::getTypeLabel
		).addString(
			"privateUrl", GroupWrapper::getPrivateURL
		).addString(
			"publicUrl", GroupWrapper::getPublicURL
		).build();
	}

	private GroupWrapper _getGroupWrapper(
			long groupId, ThemeDisplay themeDisplay)
		throws PortalException {

		return new GroupWrapper(_groupService.getGroup(groupId), themeDisplay);
	}

	private PageItems<GroupWrapper> _getPageItems(
			Pagination pagination, ThemeDisplay themeDisplay)
		throws PortalException {

		List<GroupWrapper> groupWrappers = Stream.of(
			_groupService.getGroups(
				themeDisplay.getCompanyId(), 0, true,
				pagination.getStartPosition(), pagination.getEndPosition())
		).flatMap(
			List::stream
		).map(
			group -> new GroupWrapper(group, themeDisplay)
		).collect(
			Collectors.toList()
		);
		int count = _groupService.getGroupsCount(
			themeDisplay.getCompanyId(), 0, true);

		return new PageItems<>(groupWrappers, count);
	}

	private Long _getParentGroupId(Group group) {
		if (group.getParentGroupId() != 0L) {
			return group.getParentGroupId();
		}

		return null;
	}

	@Reference
	private GroupService _groupService;

}